/*-
 * Copyright (c) 2000 Edward Brocklesby.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

#include "getusers.h"

struct server_reply *
wait_single_numeric(int fd, char *result, char *numeric)
{
	struct wait_tab numerics[] = {
		{ numeric, result},
		{ NULL, NULL}
	};

	return wait_numeric(fd, numerics);
}

static int
check_numeric(struct wait_tab numerics[], int argc, char *argv[MAX_ARG])
{
	int i;

	for (i = 0; numerics[i].numeric != NULL; i++)
		if (!strcmp(numerics[i].numeric, argv[1])) {
			fprintf(stderr, " %s", numerics[i].result);
			fflush(stderr);
			return 1;
		}
	return 0;
}

static int
check_ping(int fd, int argc, char *argv[MAX_ARG])
{
	if (!strcmp(argv[0], "PING")) {
		sock_send(fd, "PONG :%s", argv[1]);
		fprintf(stderr, " PONG");
		return 1;
	}
	return 0;
}

static void
check_error(int fd, int argc, char *argv[MAX_ARG])
{
	if (!strcmp(argv[0], "ERROR")) {
		fprintf(stderr, "\nerror  = %s\n", argv[1]);
		exit(1);
	}
}

struct server_reply *
wait_numeric(int fd, struct wait_tab numerics[])
{
	static struct server_reply reply;
	static int argc;
	static char *argv[MAX_ARG];
	char *line;
	
	for (;;) {
		line = sock_read_line(fd);
		if (*line == ':')
			line++;

		create_vector(&argc, argv, line);

		check_error(fd, argc, argv);

		if (check_ping(fd, argc, argv))
			break;

		if (check_numeric(numerics, argc, argv))
			break;
	}

	reply.argc = argc;
	reply.argv = argv;

	return &reply;
}


void format_lusers_hybrid(struct server_reply *reply, int mflag, char **dest, int special)
{
	if (mflag) {
		int lac; 
		char *lav[MAX_ARG];

		create_raw_vector(&lac, lav, reply->argv[3]);

		if (special & HYBRID_FORMAT) 
			asprintf(dest, "%s", lav[3]);
		else if (special & ULTIMATE_FORMAT)
			asprintf(dest, "%s", lav[0]);

		free_vector(lav, lac);
	} else {
		asprintf(dest, "%s", reply->argv[3]);
	}
}

void
format_lusers_irc28(struct server_reply *reply, int mflag, char **dest, int special)
{
		int lac;
		char *lav[MAX_ARG];

		create_raw_vector(&lac, lav, reply->argv[3]);

		if (special & IRC28_LOCAL) {
			/* :pacific.nsw.au.austnet.org 255 Hwy-T :I have 166 clients and 1 servers */
			if (mflag)
				asprintf(dest, "%d", atoi(lav[2]));
			else
				asprintf(dest, "Current local users: %d Max: 0", atoi(lav[2]));
		} else if (special & IRC28_GLOBAL) {
			/* :irc.elric.net 251 Hwy-T :There are 81 users and 389 invisible on 13 servers */
			int users = (special & IRC29_FORMAT) ? atoi(lav[2]) : atoi(lav[2]) + atoi(lav[5]);

			if (mflag)
				asprintf(dest, "%d", users);
			else
				asprintf(dest, "Current global users: %d Max: 0", users);
		} else {
			fprintf(stderr, "\nerror  = internal error\n");
			exit(1);
		}
}

void
format_lusers(struct server_reply *reply, int mflag, char **dest, int special)
{
	if (special & (HYBRID_FORMAT | ULTIMATE_FORMAT))
		format_lusers_hybrid(reply, mflag, dest, special);
	else if (special & (IRC28_FORMAT | IRC29_FORMAT))
		format_lusers_irc28(reply, mflag, dest, special);
	else {
		fprintf(stderr, "\nerror  = internal error\n");
		exit(1);
	}
}

struct {
	char *name;
	int type;
} stypes[] = {
	{"2.8/hybrid", 	STYPE_HYBRID},
	{"2.8.2+CS", 	STYPE_CS},
	{"2.8/comstud",	STYPE_COMSTUD},
	{"bahamut", 	STYPE_BAHAMUT},
	{"Elite",	STYPE_ELITE},
	{"Unreal",	STYPE_UNREAL},
	{"UltimateIRCd",STYPE_ULTIMATE},
	{"BeyondIRC",	STYPE_BEYOND},
	{"austhex",	STYPE_AUSTHEX},
	{"Xnet",	STYPE_XNET},
	{"glx",		STYPE_GLX},
	{"ConferenceRoom", STYPE_CR},
	{"relic",	STYPE_RELIC},
	{"dal",		STYPE_DAL},
	{"sor",		STYPE_SOR},
	{"nn",		STYPE_NEWNET},
	{"cn",		STYPE_CHATNET},
	{"2.8",		STYPE_2_8},
	{"2.9",		STYPE_2_9},
	{"2.10",	STYPE_2_10},
	{"u2",		STYPE_IRCU},
	{NULL, 0}
};

int
find_version(struct server_reply *reply)
{
	int i;

	for (i = 0; stypes[i].name; i++)
		if (!strncasecmp(reply->argv[3], stypes[i].name, strlen(stypes[i].name)))
			return stypes[i].type;

	return STYPE_UNKNOWN;
}

