/*-
 * Copyright (c) 2000 Edward Brocklesby.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* small program to connect to an irc server and obtain the current
   usage statistics */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "getusers.h"
#include "config.h"

static void process_server(char *);
static void initialise(int *, char ***, char *);

static void
usage(char *progname)
{
	fprintf(stderr, "Usage: %s [-mlg] [-p port] [-t target] [-v vhost] [-f file] [-o nick] [-P pass] [server[:port][, ...]]\n", progname);
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "\tserver           server to connect to, if not specified in config file\n");
	fprintf(stderr, "\t-m               output stats in MRTG-ready format\n");
	fprintf(stderr, "\t-p port          specify port to connect to\n");
	fprintf(stderr, "\t-t target        output stats for specified server\n");
	fprintf(stderr, "\t-v vhost         connect from a certain virtual host\n");
	fprintf(stderr, "\t-f file          specify configuration file\n");
	fprintf(stderr, "\t-l               show local users only\n");
	fprintf(stderr, "\t-g               show global users only\n");
	fprintf(stderr, "\t-o nick          oper upon connect\n");
	fprintf(stderr, "\t-P pass          password to use for -o\n");
	fprintf(stderr, "At least one server or configuration file must be given.\n");
	exit(1);
}

int
main(int argc, char **argv)
{
	int i;
	char *progname = argv[0];
	char **servers = NULL;

	initialise(&argc, &argv, progname);

	if ((!config.server && argc != 1) || (config.server && argc > 1))
		usage(progname);

	if (!config.server && argc == 1)
		config.server = argv[0];

	{
		char *ap;
		
		i = 0;

		while ((ap = strsep(&config.server, ",")) != NULL) {
			if (*ap != '\0') {
				servers = realloc(servers, sizeof(char *) * i);
				servers[i] = ap;
				i++;
			}
		}
	}

	for (i = 0; servers[i]; i++) 
		process_server(servers[i]);

	return 0;
}

static void
initialise(int *argc, char ***argv, char *progname)
{
	int c;

        memset(&config, 0, sizeof(config));
        config.port = DEFAULT_PORT;

#ifdef DEFAULT_CONFIG
        parse_config(DEFAULT_CONFIG);
#endif

        while ((c = getopt(*argc, *argv, "mp:t:v:f:lgo:P:")) != -1)
                switch(c) {
                        case 'p':
                                config.port = atoi(optarg);
                                break;
                        case 'm':
                                config.mrtg = 1;
                                break;
                        case 't':
                                config.target = malloc(strlen(optarg) + 1);
                                strcpy(config.target, optarg);
                                break;
                        case 'v':
                                config.vhost = malloc(strlen(optarg) + 1);
                                strcpy(config.vhost, optarg);
                                break;
                        case 'f':
                                if (parse_config(optarg) == -1) {
                                        fprintf(stderr, "error  = %s: %s\n", optarg, strerror(errno));
                                        exit(1);
                                }
                                break;
			case 'l':
				config.lo = 1;
				break;
			case 'g':
				config.go = 1;
				break;
			case 'o':
				config.oper = 1;
				config.opernick = malloc(strlen(optarg) + 1);
				strcpy(config.opernick, optarg);
				break;
			case 'P':
				config.operpass = malloc(strlen(optarg) + 1);
				strcpy(config.operpass, optarg);
				break;
                        case '?':
                        default:
                                fprintf(stderr, "Unknown option %c\n", c);
                                usage(progname);
                                /*NORETURN*/
                }

        *argc -= optind;
        *argv += optind;
}

static void
process_server(char *server)
{
	int fd, version, special = 0;
	char *lrep1, *lrep2, *s;
	struct server_reply *reply;
	int port;

	s = strchr(server, ':');
	if (s) {
		*s++ = '\0';
		port = atoi(s);
	} else
		port = config.port;

	fprintf(stderr, "server = %s\n", server);
	fprintf(stderr, "port   = %d\n", port);

	if (config.target)
		fprintf(stderr, "target = %s\n", config.target);
	if (config.vhost)
		fprintf(stderr, "vhost  = %s\n", config.vhost);

	if (config.oper)
		fprintf(stderr, "oper   = yes [%s]\n", config.opernick);

	fprintf (stderr, "status =");
	fflush(stderr);
	
	fd = sock_connect(server, port, config.vhost);

	if (fd == -1) {
		fprintf(stderr, "\nerror = %s\n", strerror(errno));
		exit(8);
	}

	fprintf(stderr, " CONNECT");
	fflush(stderr);

	wait_single_numeric(fd, "REGISTER", "001");

	{	struct wait_tab ltab[] = {
						 {"376", "MOTD"},
						 {"422", "NOMOTD"},
						 {NULL, NULL}
					 };

		wait_numeric(fd, ltab);
	}

	/* if we're meant to oper, do it */
	if (config.oper) {
		sock_send(fd, "OPER %s %s", config.opernick, config.operpass);
	}

	/* find the remote server version */
	sock_send(fd, "VERSION");
	reply = wait_single_numeric(fd, "VERSION", "351");

	version = find_version(reply);
	free_vector(reply->argv, reply->argc);

	switch(version) {
		case STYPE_HYBRID:
		case STYPE_BAHAMUT:
		case STYPE_UNREAL:
		case STYPE_NEWNET:
		case STYPE_CR:
		case STYPE_BEYOND:
		case STYPE_RELIC:
			special |= HYBRID_FORMAT;
			break;
		case STYPE_ULTIMATE:
			special |= ULTIMATE_FORMAT;
			break;
		case STYPE_2_9:
		case STYPE_2_10:
			special |= IRC29_FORMAT;
			/*FALLTHROUGH*/
		case STYPE_AUSTHEX:
		case STYPE_XNET:
		case STYPE_GLX:
		case STYPE_DAL:
		case STYPE_SOR:
		case STYPE_COMSTUD:
		case STYPE_CS:
		case STYPE_2_8:
		case STYPE_IRCU:
			special |= IRC28_FORMAT | SWAP_OUTPUT;
			break;
		case STYPE_ELITE:
#if 0
			special |= STRIP_ONE;
#endif
			fprintf(stderr, "\nerror  = remote server type not supported\n");
			exit(1);
		default:
		case STYPE_UNKNOWN:
			fprintf(stderr, "\nerror  = remote server version is unknown\n");
			exit(1);
	}

	/* send the LUSERS command */
	if (config.target)
		sock_send(fd, "LUSERS * %s", config.target);
	else
		sock_send(fd, "LUSERS");

	if (special & (HYBRID_FORMAT | ULTIMATE_FORMAT)) {
		reply = wait_single_numeric(fd, "LUSERS_HYBRID_1", "265");
		format_lusers(reply, config.mrtg, &lrep1, special);
		free_vector(reply->argv, reply->argc);

		reply = wait_single_numeric(fd, "LUSERS_HYBRID_2", "266");
		format_lusers(reply, config.mrtg, &lrep2, special);
		free_vector(reply->argv, reply->argc);
	} else if (special & IRC28_FORMAT) {
		reply = wait_single_numeric(fd, "LUSERS_28_1", "251");
		format_lusers(reply, config.mrtg, &lrep1, special | IRC28_GLOBAL);
		free_vector(reply->argv, reply->argc);

		reply = wait_single_numeric(fd, "LUSERS_28_2", "255");
		format_lusers(reply, config.mrtg, &lrep2, special | IRC28_LOCAL);
		free_vector(reply->argv, reply->argc);
	} else {
		fprintf(stderr, "\nerror = internal error\n");
		exit(1);
	}

	sock_send(fd, "QUIT :done");

	fputs("\n", stderr);

	{
		char *llrep1 = lrep1, *llrep2 = lrep2;


		if (config.lo || config.go)
		{
			llrep1 = llrep2 = config.go ? lrep2 : lrep1;
		}

		if (special & SWAP_OUTPUT)
			printf("%s\n%s\n", llrep2, llrep1);
		else
			printf("%s\n%s\n", llrep1, llrep2);
	}

	return;
}

